// RDBurnFileGenerator.cpp : Defines the entry point for the console application.
// (c) 2002 Ch. Krger

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <conio.h>

#define EPROMSIZE 65536

unsigned char eprombuffer[EPROMSIZE];
unsigned char* writepos;

const unsigned char* eobuffer = eprombuffer+EPROMSIZE;

const int SECSIZE = 128;
int	SECTORS;

char  extension[5];
char* extensionpos;
char		   destfilename[256];
unsigned char autooption=0;

bool
IsSectorEmpty(char* sector)
{
	int i;
	for (i=0; i<SECSIZE;i++,sector++)
	{
		if (*sector)
			return false;
	}
	return true;
}


long
FileLength(FILE* file)
{
	long pos,length;
	
	pos=ftell(file);
	fseek(file,0,SEEK_END);
	length = ftell(file);
	fseek(file,pos,SEEK_SET);
	return length;
}


void
FlushBufferToFile(void)
{
	FILE* destfile = fopen(destfilename,"wb");
	fwrite(eprombuffer, EPROMSIZE, 1, destfile);
	fclose(destfile);
}


void
FlushAndPrepareNewBuffer(int actsector)
{
	unsigned char* headerpos = eprombuffer;

	// Nachtragen: Multipler Modus & Endsektor...
	
	*headerpos++ = 129;		// multipler EPROM Modus...
	headerpos++;			// start berspringen!
	headerpos++;

	unsigned char highSecCount = (actsector-1) >> 8;
	unsigned char lowSecCount  = (actsector-1) - (highSecCount<<8);

	*headerpos++ = lowSecCount;
	*headerpos++ = highSecCount;

	strcpy(extensionpos, extension);

	printf("\nWriting file...\n");
	FlushBufferToFile();

	// Buffer wieder vorbereiten...
	strcpy(extension,".EP2");

	writepos = eprombuffer;		// Buffer wieder von vorne nutzen...

	*writepos++ = 0;			// letze EPROM-Datei (sonst 1...)

	highSecCount = actsector >> 8;
	lowSecCount  = actsector - (highSecCount<<8);

	*writepos++ = lowSecCount;	// start sektor eintragen
	*writepos++ = highSecCount; // (L/H)
	*writepos++;				// end sektor wird nachgetragen...
	*writepos++;
	*writepos++ = 10;			// Offset zum Datenanfang (ohne Name immer 10)
	*writepos++ = 0;			// (?)
	*writepos++ = autooption;	// auto option flag..
	*writepos++ = 0;			// (?)
	*writepos++ = 0;			// (?)
}


void
FlushBuffer(void)
{
	unsigned char* headerpos = eprombuffer;

	// Endsektor eintragen...

	headerpos += 3;		// position zeigt auf End-Sector...
	
	unsigned char highSecCount = SECTORS >> 8;
	unsigned char lowSecCount  = SECTORS - (highSecCount<<8);

	*headerpos++ = lowSecCount;
	*headerpos++ = highSecCount;

	strcpy(extensionpos, extension);
	printf("\nWriting file...\n");
	FlushBufferToFile();	
}


int
main(int argc, char* argv[])
{
	if (argc=! 2)
		printf("Usage: BurnFileGen srcfile.xfd");


	FILE* srcfile = fopen(argv[1],"rb");
	SECTORS = FileLength(srcfile)/SECSIZE;
	
	char* srcfilebuffer = (char*)calloc(SECTORS,SECSIZE);

	if (!srcfile)
	{
		printf("Cannot open source file!\n");
		free(srcfilebuffer);
		return 0;
	}

	fread(srcfilebuffer,SECTORS,SECSIZE,srcfile);
	fclose(srcfile);

	strcpy(destfilename, argv[1]);
	
	if (!(extensionpos=strrchr(destfilename,'.')))
	{
		printf("Invalid source file name!\n");
		free(srcfilebuffer);
		return 0;
	}

	strcpy(extension,".EP1");	// Setup base extension...

	writepos = eprombuffer;		// Setup start..
	*writepos++ = 128;	// EPROM-Typ default: single (multiple wird nachgetragen..)
	*writepos++	= 1;	// Startsektor
	*writepos++	= 0;	// (Low/High)
	writepos++;			// Endsektor wird nachgetragen!
	writepos++;
	
	int	 namelen = 80;
	char epromname[80];

	while(namelen>20)
	{
		printf("Eprom name (20 char. max.):\n");
		gets(epromname);
		namelen=strlen(epromname);
	}

	printf("Automatic 'option' mode (Y/N)?\n");
	int answer = _getch();
	_putch(answer);
	if (answer=='Y' || answer=='y')
		autooption = 255;

	*writepos++ = namelen+10;	// offset zu den daten...
	*writepos++ = 0;			// (?)
	*writepos++ = autooption;	// Auto Option: 255 = JA, 0 = NEIN
	*writepos++ = 0;			// (?)
	*writepos++ = 0;			// (?)

	int i;
	for(i=0; i<namelen; i++)
	{
		char c = *(epromname+i); // ASCII nach XL-Bildschirmcode...
		if (c<97)
			c-=32;
		*writepos++ = c;
	}

	char* sectorbuffer = srcfilebuffer;

	// Sektoren bertragen...

	for (int sector=0; sector < SECTORS; sector++, sectorbuffer+=SECSIZE)
	{
		int epromSpaceLeft = eobuffer - writepos;

		if (IsSectorEmpty(sectorbuffer))
		{
			if (!epromSpaceLeft)
				FlushAndPrepareNewBuffer(sector+1); // SEKTOREN werden von 1
			*writepos++ = 128;						// an gezhlt!
		}
		else
		{
			if (epromSpaceLeft<(SECSIZE+1))
				FlushAndPrepareNewBuffer(sector+1);	// Sektoren werden von 1
			*writepos++ = 0;						// an gezhlt...
			memcpy(writepos,sectorbuffer,SECSIZE);
			writepos+=SECSIZE;
		}
	}

	FlushBuffer();

	free(srcfilebuffer);
	return 0;
}

